module GravatarFor
  GRAVATAR_BASE_URL = 'http://www.gravatar.com/avatar/'
  GRAVATAR_SECURE_BASE_URL = GRAVATAR_BASE_URL.gsub('://www', 's://secure')

  module ViewHelpers
    # Returns an HTML5 image tag to the Gravatar image for +email+.
    #
    # ==== Options
    #
    # You can add HTML attributes and modify Gravatar results using the
    # +options+
    #
    # * <tt>:size</tt>    - If no size is given; Gravatar's default
    #   of 80px by 80px is used.
    # * <tt>:default</tt> - The default URL for an image to display if the
    #   email has no Gravatar image. By default, Gravatar returns its logo if
    #   there is no image to reference. To cover all basis, you can use the
    #   following symbols to use some of Gravtar's other options:
    #     * :mm        - Mystery Man
    #     * :identicon - A geometric pattern based on an email has
    #     * :monsterid - A generated monster
    #     * :wavatar   - Computer generated faces
    #     * :retro     - 8-bit generated arcade-styled face
    #     * :blank     - A transparent PNG image
    #     * 404        - HTTP 404 response (File Not Found)
    # * <tt>:rating</tt> - Rating to request, either :g or 'g' (default);
    #   :pg or 'pg'; :r or 'r'; :x or 'x'
    # * <tt>:forcedefault</tt> - Force default image, default: false
    # * <tt>:alt</tt> - Override the HTML alternate text attribute. By default
    #   the email provided will be placed into this attribute.
    # * <tt>:ssl</tt> - Use regular or secure URL, default +false+
    # * Any additional +options+ will be passed to
    #   ActionView::Helpers::TagHelper::tag
    def gravatar_for(email, options = {})
      raise ArgumentError, 'Email must be provided' unless email && !email.strip.empty?

      base_url = options[:ssl]? GRAVATAR_SECURE_BASE_URL : GRAVATAR_BASE_URL
      options.delete(:ssl) if options.has_key?(:ssh)

      options[:alt] ||= email

      query = ''
      [:size, :default, :rating, :forcedefault].each do |key|
        if options.has_key?(key)
          query << (query.empty? ? '?' : '&')
          query << "#{key}=#{options.delete(key)}"
        end
      end

      src = base_url + Digest::MD5.hexdigest(email)
      src << query unless query.empty?

      options[:src] = src

      tag('img', options, true)
    end
  end
end
